# DSPs are associated in groups, which determine how they are packaged.
# group0 is special, in that DSPs in this group will go out with the
# Power Tools addon itself.

group0 := \
	formmail-5.0		\
	oscommerce-2.2ms2	\
	oscommerce-2.2ms2_060817	\
	postnuke-0.7.2.6 	\
	postnuke-0.7.6.4 	\
	phpnuke-7.7		\
	phpnuke-7.8		\
	phpnuke-7.9		\
	phpopenchat-3.0.0b7	

group1 := \
	xoops-2.0.13		\
	xoops-2.0.16		\
        b2-1.5.1.3		\
	cslivehelp-2.7		\
	gallery-1.5		\
	gallery-2.0.1		\
	phplist-2.10.4		\
	invboard-1.3            \
	phplinks-2.1.3.1	\
	phprojekt-4.1.1         \
	phpsupporttickets-1.7	\
	phpbb-2.0.21		\
	phpbb-2.0.22		\
	piki-1.0		\
	webcalendar-1.0.4	\
	phpopenchat-3.0.2	

retired := \
	formmail-4.2b		\
	postnuke-0.7.2.3	\
	phpbb-2.0.5		\
	phpbb-2.0.6		\
	phpbb-2.0.19		\
	phpnuke-6.8		\
	phpnuke-6.9		\
	phpnuke-7.0		\
	phpopenchat-3.0.0b2	\
	phpopenchat-3.0.0b3	\
	phpsupporttickets-1.5	\
        b2-0.6.2.1              \
        b2-0.6.2.2              \
        b2-1.2.1		\
	phplist-2.6.2		\
	phplist-2.7.2		\
	webcalendar-0.9.42      \
	webcalendar-1.0.2	\
	gallery-1.4		\
	gallery-1.4.1		\
	gallery-1.3.4		\
	invboard-1.1.2		\
	invboard-1.2            \
        xoops-2.0.3		\
	xoops-2.0.5		\
        phprojekt-4.0.4

groups := group0 group1

ifdef MAKE_RETIRED
groups += retired
endif

define make_dsp_dir
$(subst /,-,$(patsubst %/,%,$(dir $(subst -,/,$(1)))))/$(notdir $(subst -,/,$(1)))
endef

dsps := $(foreach group,$(groups),$($(strip $(group))))
dsp_dirs := $(foreach dsp,$(dsps),$(call make_dsp_dir,$(dsp)))
non_dsp_dirs := translations rpmbuild 

#
# YOU GENERALLY WILL NOT NEED TO MODIFY ANYTHING BELOW THIS LINE
#

.PHONY: bump_dapi_releases
bump_dapi_releases : make_utils/bump_dapi_release.xml $(addsuffix /dsp.xml,$(dsp_dirs))
	$(foreach dsp_xml,$(filter %/dsp.xml,$^),\
          $(XSLTPROC) $< $(dsp_xml) > $(dsp_xml).tmp &&	\
          $(MV) -f $(dsp_xml).tmp $(dsp_xml) &&) $(TRUE)

build_root := build
build_dirs := . 
source_dirs := $(dsp_dirs) $(non_dsp_dirs)
common_dirs := 

deployables_dir := usr/lib/ensim/deployables

define dep_root_dir
/$(deployables_dir)/$(dspname)
endef

define dep_install_dir
$($(dspname)_directory)/INSTALL/$(deployables_dir)/$(dspname)
endef

define dsp_source_dir
$(dep_install_dir)/source
endef

define dsp_lib_dir
$(dep_install_dir)/lib
endef

define dsp_theme_dir
$(dsp_lib_dir)/themes
endef

define dsp_source_files
$(addprefix $(dsp_source_dir)/,$($(dspname)_source_files))
endef

define dsp_lib_files
$(addprefix $(dsp_lib_dir)/,$($(dspname)_raw_lib_files))
endef

define dsp_theme_files
$(addprefix $(dsp_theme_dir)/,$($(dspname)_themes))
endef

define dsp_rpm
RPMS/i386/$(dspname)-rpm
endef

define dsp_xml
$($(dspname)_directory)/dsp.xml
endef

define dsp_install
$(dspname)_install
endef

define dsp_dAPI_files
$(addprefix $(dsp_lib_dir)/,dAPI.py dAPI.pyc)
endef

define dsp_dependencies
$(dsp_source_files) $(dsp_lib_files) $(dsp_theme_files) $(dsp_dAPI_files)
endef

ifdef BUILDING

ifdef old_dsp_rpm_location
-include delta_dsps.d
source_dirs += $(non_dsp_dirs)
endif

%.pyc : %.py
	$(call cmdcheck,$(PYTHONCOMP) $< $@ $<, $(print-header))

%/msgs.py : util/errornum.py
	$(call cmdcheck,$(CPP) -P -E -o $@ -I$(dir $(filter %/util/common_msgs.list,$^)) -include $(filter %/errornum.py,$^) $(filter %/msgs.list,$^), $(print-header))

%/msgs.pm : 
	@echo autogenerating $@
	@cat $(filter %/msgs.list,$^) |sed '/^#/d' |sed '/^ *$$/d' | awk -v quot='"' -v cpat='\\$$' '\
	BEGIN {\
	        print "#auto-generated perl error module for $(shell dirname $@)";\
	print "package msgs;";\
	print "require Exporter;";\
	printf ("our @ISA = (%sExporter%s);", quot, quot);\
	print "";\
	        varname=1;\
	        expvars="qw(";\
	}\
	{\
	if (varname) { printf ("$$%s=",$$0); varname=0;\
	                expvars =sprintf("%s $$%s",expvars,$$0); }\
	        else {\
	                if (!match($$0,cpat)) { varname=1; printf ("%s;",$$0); print "";}\
	                else { print $$0;}\
	        }\
	}\
	END {\
	        expvars =sprintf("%s)",expvars);\
	        printf ("our @EXPORT = %s;", expvars);\
	        print "";\
	        print "1;";\
	}\
	' > $@

%/dsp.d : %/dsp.xml make_utils/xml_dep util/dsp.dtd make_utils/ver_rel.xsl util/dsp_options.xsl
	$(call cmdcheck,$(filter %/make_utils/xml_dep,$^) $* $< $(filter %/util/dsp.dtd,$^) $*/INSTALL/$(deployables_dir)/$(subst /,-,$*)/lib $(filter %/make_utils/ver_rel.xsl,$^) $(filter %/util/dsp_options.xsl,$^) > $@, $(print-header))

%/msgs.d : %/msgs.list make_utils/msgs_dep
	$(call cmdcheck, $(filter %/make_utils/msgs_dep,$^) $* $< $*/INSTALL/$(deployables_dir)/$(subst /,-,$*)/lib > $@, $(print-header))

delta_dsps.d : make_utils/delta_dsps_dep make_utils/ver_rel.xsl
	$(call cmdcheck,$< $(filter %/make_utils/ver_rel.xsl,$^) $(old_dsp_rpm_location) $(filter %.xml,$^) > $@, $(print-header))

dep_suffixes += .xml .list .manifest

# These two need to be static so that they accumulate nicely
dsp_rpms :=
dsp_installs :=

endif
