import os
import pwd
import urllib
import urlparse
import deployables_lib as dl
import md5
import time
import math

dl.load_msgs_module("DSP_SHORTNAME-DSP_VERSION", symtab = globals())

dir_moves = [('DSP_SHORTNAME', '')]
file_mods = [{'path' : 'tmp/', 'create' : '', 'delete' : ''},
             {'path' : 'albums/', 'create' : '', 'delete' : ''},
             {'path' : 'config.php', 'delete' : ''},
             {'path' : '.htaccess', 'delete' : ''}]
secure_files = [('config.php', 0600, 0600),
		('.htaccess', 0644, 0644),
		('setup', 0, 0)]
instance_patches = []

pat = "^\s*\$gallery->setConfig\(\'(?P<key>\S+\w)\s*\',\s*(?P<value>['\"].*['\"])\)\s*;\s*"
sub = "$gallery->setConfig(\'%(key)s\', %(value)s);"
pat1 = "^\s*\$storeConfig\[\'(?P<key>\S+\w)\s*\'\]\s*=\s*(?P<value>['\"].*['\"])\s*;\s*"
sub1 = "$storeConfig[\'%(key)s\'] = %(value)s;"
pat2 =	"(?P<key>\w+)(?P<value>.*)"
sub2 = "%(key)s%(value)s"

def install(site, site_conf, instance_conf, msgs):
    actions = ['resolve_location', 'check_disk_quota', 'check_pass',
               'check_install_loc_owner', '']
    args_dict = {}

    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR :
        actions = ['db_create', 'db_probe','archive_unpack', 'check_index', 
                   'set_php_cgi', 'handle_security_mode',
                   'edit_conf_file','run_sql_file','exec_mysql', 'access_url']

        vars = [('data.gallery.base', dl.site_fs_jail(site, site_conf) + instance_conf['install_loc'] +'/g2data'),
		('setup.password',instance_conf['instance_pass1'])]

	password='2k^Z'+md5.md5('2k^Z%s' %instance_conf['instance_pass1']).hexdigest()
	inst_conf={}
	inst_conf['instance_pass1'] = password
	inst_conf['instance_admin_email'] = instance_conf['instance_admin_email']
	sql_stmt1 = ["""
			UPDATE g2_User SET g_hashedPassword = '%(instance_pass1)s',
			g_email = '%(instance_admin_email)s' WHERE g_userName = 'admin' 
		     """ % inst_conf]

        args_dict = {'archive_unpack' : {'dir_moves':dir_moves,'file_mods':file_mods},
	    	     'run_sql_file' : {'file' : 'dump_database.sql'},
                     'handle_security_mode' : {'files': secure_files},
         	     'edit_conf_file' : { 'file': 'config.php','lang': 'php',
                                  'pat': pat, 'sub': sub, 'vars': vars },
                     'exec_mysql' : {'sql' : sql_stmt1},'access_url' : {'user' : 'admin'}}
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)
        
        actions = ['edit_conf_file']

	vars = [('database', instance_conf['dbname']),
                ('username', site_conf['mysql']['dbaseadmin']),
                ('password', instance_conf['dbpass'])]

        args_dict = {'edit_conf_file' : { 'file': 'config.php','lang': 'php','pat': pat1, 'sub': sub1, 'vars': vars }}

        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

	mod_time = int(math.ceil(time.time()))
	config_data = """modules/core/classes/GalleryUser.class|O:11:"galleryuser":15:{s:9:"_userName";s:5:"admin";s:9:"_fullName";s:21:"Gallery Administrator";s:15:"_hashedPassword";s:%(pass_len)s:"%(password)s";s:6:"_email";s:%(email_len)s:"%(emailid)s";s:9:"_language";N;s:3:"_id";i:6;s:18:"_creationTimestamp";i:1131528006;s:11:"_isLinkable";i:0;s:7:"_linkId";N;s:13:"_linkedEntity";N;s:22:"_modificationTimestamp";i:%(mod_time)s;s:13:"_serialNumber";i:1;s:11:"_entityType";s:11:"GalleryUser";s:15:"_onLoadHandlers";N;s:17:"_persistentStatus";O:8:"stdClass":3:{s:5:"flags";i:0;s:8:"modified";a:0:{}s:13:"originalValue";a:0:{}}}"""
	local_config_data = {'password' : password, 'emailid' : instance_conf['instance_admin_email'], 'mod_time' : mod_time, 'pass_len' : len(password), 'email_len' : len(instance_conf['instance_admin_email'])}
	vars = [('modules',config_data % local_config_data)]
	sub2 = config_data % local_config_data
	actions = ['edit_conf_file']
	
	args_dict = {'edit_conf_file' : { 'file': 'g2data/cache/entity/0/0/6.inc','lang': 'php','pat': pat2, 'sub': sub2, 'vars': vars }}
	dl.do_install_actions(site, site_conf, instance_conf, msgs, globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    # Edit the Configuration file.                                       
    old_domain = old_site_conf['siteinfo']['domain']
    new_domain = site_conf['siteinfo']['domain']
    def url_sub(key, url):
        if context == dl.CONTEXT_SITE_EDIT:
            return dl.url_net_sub(url, new_domain, old_domain)
        else:
            return dl.url_sub(url, instance_conf['url_loc'],
                              old_instance_conf['url_loc'])
    def rewrite_sub(key, value):
        return dl.url_path_sub(value, instance_conf['url_loc'],
                               old_instance_conf['url_loc'])

    old_loc = "%s%s" % (dl.site_fs_jail(site, old_site_conf),
                        old_instance_conf['install_loc'])
    new_loc = "%s%s" % (dl.site_fs_jail(site, site_conf),
                        instance_conf['install_loc'])
    vars = [('data.gallery.base', dl.site_fs_jail(site, site_conf) + instance_conf['install_loc'] +'/g2data')] 

    actions = ['edit_conf_file']
    
    args_dict = {'edit_conf_file' : { 'file': 'config.php','lang': 'php','pat': pat, 'sub': sub, 'vars': vars }}

    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf,msgs,
                                  globals(), actions, args_dict )

def upgrade(site, site_conf, instance_conf, old_instance_conf, msgs):
    if old_instance_conf['shortname'] == 'gallery' and \
	   ( old_instance_conf['version'] == '1.5' ):
        instance_conf['syncfs'] = '1'
        instance_conf['overwritefiles'] = '1'

        uid, gid = dl.get_site_user_ids(site, instance_conf['username'])
        
        # Need to temporarily loosen the setup directory permissions
        # for backup; we'll lock it back down after the upgrade is done
        path = dl.site_fs(site) + instance_conf['install_loc'] + '/setup'
        if instance_conf['do_backup'] == '1' and os.path.exists(path):
            dl.chmodfile(path, uid, gid, 0500)
        
        actions = ['do_backup', '', 'archive_unpack',
                   'handle_security_mode']
        args_dict = {
            'archive_unpack': { 'dir_moves' : dir_moves },
            'handle_security_mode' : { 'files': secure_files }
            }
        dl.do_upgrade_actions(site, site_conf, instance_conf,
                              old_instance_conf, msgs,
                              globals(), actions, args_dict)
        if msgs.get_severity() != msgs.ERROR:
            # Upgrade/Add configuration directives
            path = dl.site_fs(site) + instance_conf['install_loc'] + \
                   '/config.php'
            conf_data = dl.readfile(path, uid, gid)
            new_conf_data = []
            last_line = None
            # Slightly ugly trick for delaying the write by one
            # loop, allowing me to neatly catch the end of the file
            for line in conf_data.split('\n'):
                if not line:
                    # skip blank lines
                    continue
                elif line.find('$gallery->app->config_version') != -1:
                    # Replacing this line...
                    line = "$gallery->app->config_version = '91';"
                if last_line != None:
                    new_conf_data.append(last_line)
                last_line = line
            else:
                # end of the file; add the new options
                new_conf_data.extend([\
		'$gallery->app->feature["zip"] = 1;',
		'$gallery->app->feature["rewrite"] = 1;',
		'$gallery->app->feature["mirror"] = 0;',
		'$gallery->app->galleryTitle = "Gallery";',
		'$gallery->app->useIcons = "no";',
		'$gallery->app->skinname = "none";',
		'$gallery->app->uploadMode = "form";',
		'$gallery->app->graphics = "ImageMagick";',
		'$gallery->app->pnmDir = "/usr/bin";',
		'$gallery->app->pnmtojpeg = "pnmtojpeg";',
		'$gallery->app->pnmcomp = "pnmcomp";',
		'$gallery->app->autorotate = "yes";',
		'$gallery->app->jpegImageQuality = "90";',
		'$gallery->app->albumTreeDepth = "1000";',
		'$gallery->app->highlight_size = "200";',
		'$gallery->app->showOwners = "no";',
		'$gallery->app->albumsPerPage = "5";',
		'$gallery->app->showSearchEngine = "yes";',
		'$gallery->app->slowPhotoCount = "no";',
		'$gallery->app->gallery_thumb_frame_style = "simple_book";',
		'$gallery->app->zipinfo = "/usr/bin/zipinfo";',
		'$gallery->app->unzip = "/usr/bin/unzip";',
		'$gallery->app->use_jpegtran = "/usr/bin/jpegtran";',
		'$gallery->app->default_language = "en_US";',
		'$gallery->app->ML_mode = "0";',
		'$gallery->app->available_lang[] = "en_US";',
		'$gallery->app->show_flags = "no";',
		'$gallery->app->dateString = "%%x";',
		'$gallery->app->dateTimeString = "%%c";',
		'$gallery->app->emailOn = "no";',
		'$gallery->app->emailSubjPrefix = "[Gallery]";',
		'$gallery->app->selfReg = "no";',
		'$gallery->app->selfRegCreate = "no";',
		'$gallery->app->multiple_create = "no";',
		'$gallery->app->adminCommentsEmail = "no";',
		'$gallery->app->adminOtherChangesEmail = "no";',
		'$gallery->app->useOtherSMTP = "no";',
		'$gallery->app->smtpHost = "localhost";',
		'$gallery->app->smtpFromHost = "";',
		'$gallery->app->smtpPort = "25";',
		'$gallery->app->gallery_slideshow_type = "off";',
		'$gallery->app->gallery_slideshow_length = "20";',
		'$gallery->app->gallery_slideshow_loop = "yes";',
		'$gallery->app->slideshowMode = "high";',
		'$gallery->app->comments_enabled = "yes";',
		'$gallery->app->comments_indication = "photos";',
		'$gallery->app->comments_indication_verbose = "no";',
		'$gallery->app->comments_anonymous = "no";',
		'$gallery->app->comments_display_name = "!!FULLNAME!! (!!USERNAME!!)";',
		'$gallery->app->comments_addType = "popup";',
		'$gallery->app->comments_length = "300";',
		'$gallery->app->comments_overview_for_all = "no";',
		'$gallery->app->watermarkSizes = "0";',
		'$gallery->app->stats_viewsCacheOn = "0";',
		'$gallery->app->stats_viewsCacheExpireSecs = "60";',
		'$gallery->app->stats_commentsCacheOn = "0";',
		'$gallery->app->stats_commentsCacheExpireSecs = "600";',
		'$gallery->app->stats_dateCacheOn = "0";',
		'$gallery->app->stats_dateCacheExpireSecs = "-1";',
		'$gallery->app->stats_votesCacheOn = "0";',
		'$gallery->app->stats_votesCacheExpireSecs = "3600";',
		'$gallery->app->stats_ratingsCacheOn = "0";',
		'$gallery->app->stats_ratingsCacheExpireSecs = "3600";',
		'$gallery->app->stats_cDateCacheOn = "0";',
		'$gallery->app->stats_cDateCacheExpireSecs = "-1";',
		'$gallery->app->debug = "no";',
		'$gallery->app->skipRegisterGlobals = "no";',
		'$gallery->app->timeLimit = "30";',
		'$gallery->app->blockRandomCache = "86400";',
		'$gallery->app->blockRandomAttempts = "2";',
		'$gallery->app->cacheExif = "no";',
		'$gallery->app->devMode = "no";',
		'$gallery->app->useSyslog = "no";',
		'$gallery->app->use_flock = "yes";',
		'$gallery->app->expectedExecStatus = "0";',
		'$gallery->app->sessionVar = "gallery_session";',
		'$gallery->app->rssEnabled = "yes";',
		'$gallery->app->rssMode = "basic";',
		'$gallery->app->rssMaxAlbums = "25";',
		'$gallery->app->rssVisibleOnly = "yes";',
		'$gallery->app->rssDCDate = "no";',
		'$gallery->app->rssBigPhoto = "no";',
		'$gallery->app->rssPhotoTag = "yes";',
		'$gallery->app->default["cols"] = "3";',
		'$gallery->app->default["rows"] = "3";',
		'$gallery->app->default["bordercolor"] = "black";',
		'$gallery->app->default["border"] = "1";',
		'$gallery->app->default["font"] = "arial";',
		'$gallery->app->default["thumb_size"] = "150";',
		'$gallery->app->default["resize_size"] = "640";',
		'$gallery->app->default["resize_file_size"] = "0";',
		'$gallery->app->default["max_size"] = "off";',
		'$gallery->app->default["max_file_size"] = "0";',
		'$gallery->app->default["useOriginalFileNames"] = "yes";',
		'$gallery->app->default["add_to_beginning"] = "no";',
		'$gallery->app->default["fit_to_window"] = "no";',
		'$gallery->app->default["use_fullOnly"] = "no";',
		'$gallery->app->default["print_photos"]["photoaccess"]["checked"] = "checked";',
		'$gallery->app->default["print_photos"]["shutterfly"]["checked"] = "checked";',
		'$gallery->app->default["print_photos"]["fotoserve"]["checked"] = "checked";',
		'$gallery->app->default["print_photos"]["fotokasten"]["checked"] = "checked";',
		'$gallery->app->default["print_photos"]["mpush"]["checked"] = "checked";',
		'$gallery->app->default["mPUSHAccount"] = "gallery";',
		'$gallery->app->default["returnto"] = "yes";',
		'$gallery->app->default["defaultPerms"] = "everybody";',
		'$gallery->app->default["display_clicks"] = "yes";',
		'$gallery->app->default["extra_fields"] = "Description";',
		'$gallery->app->default["showDimensions"] = "no";',
		'$gallery->app->default["item_owner_modify"] = "yes";',
		'$gallery->app->default["item_owner_delete"] = "yes";',
		'$gallery->app->default["item_owner_display"] = "no";',
		'$gallery->app->default["voter_class"] = "Nobody";',
		'$gallery->app->default["poll_type"] = "critique";',
		'$gallery->app->default["poll_scale"] = "3";',
		'$gallery->app->default["poll_hint"] = "Vote for this image";',
		'$gallery->app->default["poll_show_results"] = "no";',
		'$gallery->app->default["poll_num_results"] = "3";',
		'$gallery->app->default["poll_orientation"] = "vertical";',
		'$gallery->app->default["poll_nv_pairs"][0]["name"] = "Excellent";',
		'$gallery->app->default["poll_nv_pairs"][0]["value"] = "5";',
		'$gallery->app->default["poll_nv_pairs"][1]["name"] = "Very Good";',
		'$gallery->app->default["poll_nv_pairs"][1]["value"] = "4";',
		'$gallery->app->default["poll_nv_pairs"][2]["name"] = "Good";',
		'$gallery->app->default["poll_nv_pairs"][2]["value"] = "3";',
		'$gallery->app->default["poll_nv_pairs"][3]["name"] = "Average";',
		'$gallery->app->default["poll_nv_pairs"][3]["value"] = "2";',
		'$gallery->app->default["poll_nv_pairs"][4]["name"] = "Poor";',
		'$gallery->app->default["poll_nv_pairs"][4]["value"] = "1";',
		'$gallery->app->default["poll_nv_pairs"][5]["name"] = "";',
		'$gallery->app->default["poll_nv_pairs"][5]["value"] = "";',
		'$gallery->app->default["poll_nv_pairs"][6]["name"] = "";',
		'$gallery->app->default["poll_nv_pairs"][6]["value"] = "";',
		'$gallery->app->default["poll_nv_pairs"][7]["name"] = "";',
		'$gallery->app->default["poll_nv_pairs"][7]["value"] = "";',
		'$gallery->app->default["poll_nv_pairs"][8]["name"] = "";',
		'$gallery->app->default["poll_nv_pairs"][8]["value"] = "";',
		'$gallery->app->default["slideshow_type"] = "ordered";',
		'$gallery->app->default["slideshow_recursive"] = "no";',
		'$gallery->app->default["slideshow_loop"] = "yes";',
		'$gallery->app->default["slideshow_length"] = "0";',
		'$gallery->app->default["album_frame"] = "simple_book";',
		'$gallery->app->default["thumb_frame"] = "solid";',
		'$gallery->app->default["image_frame"] = "solid";',	
					])
                new_conf_data.append(last_line)
            dl.writefile(path, uid, gid, "\n".join(new_conf_data))

            # Upgrade the user database
            actions = ['exec_php_script']
	    #for upgrade from 1.4.1 to 1.5, gallery users are at version=5 so in init.php
	    # upgrade_users.php is not included which is required to upgrade the users.
	    # Hence the "Users upgraded successfully" string which is required to check the success 
	    # of the operation will never come. To overcome this success_re is made blank.		
	    if old_instance_conf['version'] == '1.4.1':	
	            args_dict = {
        	        'exec_php_script': {'script' : 'init.php',
                	                    'args' : {},
                        	            'success_re':
                                	    ''}
	                }
	    else:
                    args_dict = {
                        'exec_php_script': {'script' : 'init.php',
                                            'args' : {},
                                            'success_re':
                                            '.*Users upgraded successfully'}
                        }		   		
            dl.do_upgrade_actions(site, site_conf, instance_conf,
                                  old_instance_conf, msgs,
                                  globals(), actions, args_dict)           

		
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves,
                           file_mods = file_mods)

def get_instance_configuration(context, site, site_conf, new_instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(new_instance_conf['shortname'],
                                          new_instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                             new_instance_conf, old_instance_conf,
                             ['dbname','username', 'url_net', 'url_path',
                              'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    tests = [{'disp' : 'Logging in to gallery (with bad password)',
              'test' : 'login'},
             {'disp' : 'Verifying expected page contents',
              'test' : 'check_output2'}
             ]

    ret = dl.regression_check_index(action, index, test_data,
                                    '<!-- album table begin -->')
    if ret == '':
        index = index - test_data['regression_check_index']
        if action == dl.REGRESSION_GET_TEST_STRING:
            if index >= len(tests):
                return ''
            else:
                return tests[index]['disp']
        else:
            status = -1
            if tests[index]['test'] == 'login':
                url = dl.add_slash(test_data['url_loc']) + 'login.php'
                opts = [('-F', 'uname=admin'), ('-F', 'gallerypassword=ensim2'),
                        ('-F', 'login=Login')]
                status, test_data['output2'] = dl.exec_curl_cmd(url, opts)
            elif tests[index]['test'] == 'check_output2':
                test_data['success_re2'] = 'Invalid username or password' 
                status = dl.check_output(test_data['output2'],
                                         test_data['success_re2'])
            return status
    else:
        return ret
