import deployables_lib as dl
import copy

dl.load_msgs_module("phpnuke-7.9", symtab = globals())

dir_moves = [('html', ''), ('', 'archive')]
patches = []
upgrade_patches = []
instance_patches = []

pat = '^\s*\$(?P<key>\S+)\s*=\s*(?P<value>["\'].*[\'"])\s*;\s*$'
sub = '$%(key)s = %(value)s;'

def install(site, site_conf, instance_conf, msgs):

    # XXX need to change sitekey in config.php
    vars = [('dbhost', dl.get_mysql_host(site)),
            ('dbuname', site_conf['mysql']['dbaseadmin']),
            ('dbpass',  instance_conf['dbpass']),
            ('dbname',  instance_conf['dbname'])]

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '', 'db_create', 
	       'db_probe', 'archive_unpack',
               'check_index', 'run_sql_file', 'edit_conf_file', 
	       'exec_php_script', 'handle_security_mode', 'access_url','remove_files']
    args_dict = {
        'archive_unpack':       { 'dir_moves': dir_moves },
        'run_sql_file':         { 'file': 'archive/sql/nuke.sql' },
        'edit_conf_file':       { 'file': 'config.php', 'lang': 'php',
                                  'pat': pat, 'sub': sub, 'vars': vars },
        'exec_php_script':      {
            'script': 'admin.php',
            'args': { 'name': instance_conf['instance_admin'],
                      'url': 'http://%s/' % site_conf['siteinfo']['domain'],
                      'email': instance_conf['instance_admin_email'],
                      'pwd': instance_conf['instance_pass1'],
                      'user_new': '1',
                      'fop': 'create_first' },
	    'post' : '1',
            'success_re' : '.*Administration System Login*'},
        'handle_security_mode': { 'files': ['config.php'] },
        'remove_files' : { 'files' : ['upgrade77-78.php','upgrade78-79.php']}
        }
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)

def upgrade(site, site_conf, instance_conf, old_instance_conf, msgs):
	if old_instance_conf['shortname'] == 'phpnuke':
		if old_instance_conf['version'] == '7.7':
			instance_conf['syncfs'] = '1'
			instance_conf['overwritefiles'] = '1'
		
			actions = ['do_backup',
				    '', 'archive_unpack',
				    'exec_php_script',
				    'exec_php_script1',
				    'handle_security_mode',
    				    'remove_files']
			args_dict = {'archive_unpack': { 'dir_moves' : dir_moves,
					'fileinfo' :
                                  	{'excluded_files' : ['config.php']}},
				'exec_php_script' : {'script' : 'upgrade77-78.php','args' : {}},
				'exec_php_script1' : {'script' : 'upgrade78-79.php','args' : {}},
				'handle_security_mode' : { 'files': ['config.php']},
				'remove_files' : { 'files' : ['upgrade77-78.php','upgrade78-79.php']}
				}

			dl.do_upgrade_actions(site, site_conf, instance_conf,
                        	      old_instance_conf, msgs,
                          	      globals(), actions, args_dict)
			if msgs.get_severity() != msgs.ERROR:
                        	# Upgrade/Add configuration directives
               	        	uid, gid = dl.get_site_user_ids(site, instance_conf['username'])
                        	path = dl.site_fs(site) + instance_conf['install_loc'] + '/config.php'
                        	conf_data = dl.readfile(path, uid, gid)
                        	new_conf_data = ""
                        	for line in conf_data.split('\n'):
                            		if line.find('$dbhost') != -1:
                               			new_conf_data = new_conf_data + "\n" + \
                                               		"$subscription_url = \"\";\n" + \
                                               		"$admin_file = \"admin\";\n" + \
                                               		"$advanced_editor = 0;\n" + line
                            		#elif line.find('$gfx_chk') != -1:
                            		elif line.startswith('$gfx_chk'):
                               			new_conf_data = new_conf_data + "\n" + "$gfx_chk = 0;"
                            		else:
                               			new_conf_data = new_conf_data + "\n" + line
                        		dl.writefile(path, uid, gid, new_conf_data)

           	elif old_instance_conf['version'] == '7.8':
                	instance_conf['syncfs'] = '1'
          	        instance_conf['overwritefiles'] = '1'

        	        actions = ['do_backup',
                	            '', 'archive_unpack',
                   	            'exec_php_script',
                	            'handle_security_mode',
                       		    'remove_files']
                	args_dict = {
                        	'archive_unpack': { 'dir_moves' : dir_moves,
                        			'fileinfo' :
                                	  	{'excluded_files' : ['config.php']}},
                        	'exec_php_script' : {'script' : 'upgrade78-79.php','args' : {}},
                        	'handle_security_mode' : { 'files': ['config.php']},
                        	'remove_files' : { 'files' : ['upgrade77-78.php','upgrade78-79.php']}}
			dl.do_upgrade_actions(site, site_conf, instance_conf,
                              		old_instance_conf, msgs,
                              		globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
			    'fileinfo' :
			    {'excluded_files' : ['config.php']}},
        }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     
    if msgs.get_severity() != msgs.ERROR:
        vars = []
        if dl.mysql_installed(site_conf, old_site_conf):
            vars=[('dbuname', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin']),
                         ('dbname', instance_conf['dbname'],
                          old_instance_conf['dbname'])]
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('dbhost', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('dbpass', instance_conf['dbpass']))

        actions = ['edit_conf_file','handle_security_mode']
        args_dict = {
            'edit_conf_file': { 'file': 'config.php', 'lang': 'php',
                                'pat': pat, 'sub': sub, 'vars': vars },
            'handle_security_mode': { 'files': ['config.php'] },
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)     

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])

    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Welcome to PHP-Nuke!')
