import deployables_lib as dl
import urlparse
import shutil
import os
import md5
import string

dl.load_msgs_module("piki-1.0", symtab = globals())

dir_moves = [('piki-1.0', '') ]
file_mods = []
secure_files = ['LocalSettings.php']
patches = []
instance_patches = []

pat = "^\s*\$(?P<key>\S+\w)\s*=\s*(?P<value>['\"].*['\"])\s*;\s*"
sub = "$%(key)s = %(value)s;"

def install(site, site_conf, instance_conf, msgs):

    actions = ['check_disk_quota', 'check_mysql_enabled',
               'check_pass', '',
               'resolve_location', 'check_install_loc_owner']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        m = md5.new()
        m.update(instance_conf['instance_pass1'])
        encry_pass = '1-' + m.hexdigest()
        admin_user = string.capitalize(instance_conf['instance_admin'])
        sql_stmt = ["UPDATE user SET user_name = '%s', user_password = md5('%s')"
                    " WHERE user_id = '1'" % \
                    (admin_user,
                     encry_pass)]
	path = dl.site_fs(site) + instance_conf['install_loc']
	location = instance_conf['install_loc']
        path1 = '/' + location.strip('/var/www/html')
        variables =[('wgDBuser', site_conf['mysql']['dbaseadmin']),
                    ('wgDBpassword', instance_conf['dbpass']),
                    ('wgDBserver', dl.get_mysql_host(site)),
                    ('wgEmergencyContact', instance_conf['instance_admin_email']),
                    ('wgPasswordSender', instance_conf['instance_admin_email']),
                    ('wgDBname', instance_conf['dbname']),
                    ('wgScriptPath', path1),
                    ('IP', (instance_conf['install_loc'], dl.MODSTR_JAIL))]
        actions = [ 'db_create', 'db_probe', 'archive_unpack',
                    'check_index', 'run_sql_file','edit_conf_file', 'exec_mysql',
                    'handle_security_mode', 'access_url']
        args_dict = {
            'archive_unpack':       { 'dir_moves': dir_moves },
            'run_sql_file': { 'file' : 'config/database.sql' },
            'edit_conf_file': { 'file':'LocalSettings.php','lang':'php',
                                'pat' : pat, 'sub': sub, 'vars': variables },
            'exec_mysql' : {'sql' : sql_stmt},
            'handle_security_mode' : {'files' : secure_files}
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', 'require_url_loc', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes', 'handle_security_mode']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
                            'fileinfo' :
                            {'excluded_files' : ['LocalSettings.php']}},
        'handle_security_mode' : {'files' : secure_files}
        }
    if context != dl.CONTEXT_SITE_EDIT:
        # We always require the dbpass for webcalendar, since a reconfigure
        # may need to peform database updates
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        vars = []
        old_upload = "%s%s" % (dl.site_fs_jail(site, old_site_conf), instance_conf['install_loc'])
        new_upload = "%s%s" % (dl.site_fs_jail(site, site_conf), instance_conf['install_loc'])
        vars = [('IP', new_upload, old_upload)]
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('wgDBname', instance_conf['dbname'],
                          old_instance_conf['dbname']),
                         ('wgDBuser', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('wgDBserver', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('wgDBpassword', instance_conf['dbpass']))
        
        actions = ['edit_conf_file', 'apply_patch',
                   'handle_security_mode']
        args_dict = {
            'edit_conf_file':       { 'file': 'LocalSettings.php', 'lang': 'php',
                                      'pat': pat, 'sub': sub, 'vars': vars },
            'handle_security_mode': { 'files': secure_files },
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)
        
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Redirecting to ...')
