import deployables_lib as dl
import urlparse
import shutil
import os
import re

dl.load_msgs_module("webcalendar-1.0.2", symtab = globals())

dir_moves = [('WebCalendar-1.0.2', '') ]
file_mods = []
patches = []
secure_files = [('includes/settings.php', 0777, 0777)]
instance_patches = []

pat = "^\s*(?P<key>\S+\w):\s*(?P<value>.*)"
sub = "%(key)s: %(value)s"

pat1 = "^\s*\$(?P<key>\S+\w)\s*=\s*(?P<value>'.*')\s*;\s*"
sub1 = "$%(key)s = %(value)s;"

def install(site, site_conf, instance_conf, msgs):

    actions = ['check_disk_quota', 'check_mysql_enabled',
               'check_pass', '', 
	       'resolve_location', 'check_install_loc_owner']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
	uid, gid = dl.get_site_user_ids(site, instance_conf['username'])
	path = dl.site_fs(site) + instance_conf['install_loc'] + '/includes/settings.php'
        sql_stmt = ["UPDATE webcal_config SET cal_value = '%s'"
                    " WHERE cal_setting = 'email_fallback_from'" % \
                    instance_conf['instance_admin_email'],
                    "UPDATE webcal_user SET cal_login = '%s', cal_passwd = md5('%s')"
                    " WHERE cal_login = 'admin'" % \
                    (instance_conf['instance_admin'],
                     instance_conf['instance_pass1'])]
        variables = [('db_login', site_conf['mysql']['dbaseadmin']),
                     ('db_password', instance_conf['dbpass']),
                     ('db_host', dl.get_mysql_host(site)),
                     ('db_database', instance_conf['dbname']) ]
        actions = [ 'db_create', 'db_probe', 'archive_unpack',
                    'check_index', 'run_sql_file','edit_conf_file','exec_mysql',
                    'handle_security_mode', 'access_url']
        args_dict = {
            'archive_unpack':       { 'dir_moves': dir_moves },
            'run_sql_file': { 'file' : 'tables-mysql.sql' },
	    'edit_conf_file': { 'file':'includes/settings.php','lang':'apache_config',
                                'pat' : pat, 'sub': sub, 'vars': variables },
            'exec_mysql' : {'sql' : sql_stmt},
	    'handle_security_mode': {'files': secure_files }
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', 'require_url_loc', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes', 'handle_security_mode']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
                            'fileinfo' :
                            {'excluded_files' : ['includes/settings.php']}},
			'handle_security_mode': {'files': secure_files }
        }
    if context != dl.CONTEXT_SITE_EDIT:
        # We always require the dbpass for webcalendar, since a reconfigure
        # may need to peform database updates
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        vars = []
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('db_database', instance_conf['dbname'],
                          old_instance_conf['dbname']),
                         ('db_login', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('db_host', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('db_password', instance_conf['dbpass']))
        
        actions = ['edit_conf_file', 'apply_patch']
        args_dict = {
            'edit_conf_file':       { 'file': 'includes/settings.php', 'lang': 'apache_config',
                                      'pat': pat, 'sub': sub, 'vars': vars },
            'apply_patch' : {'patches' : instance_patches}
            }
        # XXX We want to do this on site edit, too, but we don't have
        # the mysql password. As it stands, changing the domain name
        # during site edit will break any webcalendar instances. The
        # workaround is to reconfigure any webcalendar instances afterwards,
        # which will trigger the url_loc change clause below.
        if context != dl.CONTEXT_SITE_EDIT:
            sql_stmt = "UPDATE webcal_config SET cal_value = '%s/'" \
                       " WHERE cal_setting = 'server_url'" % \
                       instance_conf['url_loc']
            actions.append('exec_mysql')
            args_dict['exec_mysql'] = {'sql' : sql_stmt}
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)

def upgrade(site, site_conf, instance_conf, old_instance_conf, msgs):
    if old_instance_conf['shortname'] == 'webcalendar' and \
       old_instance_conf['version'] == '0.9.42':
	instance_conf['syncfs'] = '1'
        instance_conf['overwritefiles'] = '1'
	uid, gid = dl.get_site_user_ids(site, instance_conf['username'])
        path = dl.site_fs(site) + old_instance_conf['install_loc'] + '/includes/config.php'
        conf_data = dl.readfile(path, uid, gid)
        i = 0
        dbpassword = dbdatabase = ""
        p = re.compile("\$db_\w*\s=\s\'.*\';")
        for line in p.findall(conf_data):
        	vals=re.sub("^\$\w*_\w*\s=","",line)
        	val=re.sub("^\s\'","",vals )
        	va=re.sub("\';$","",val )
        	if i == 2:
        		dbpassword = va
        	elif i == 3:
        		dbdatabase = va
        	i=i+1
	actions = ['archive_unpack', 'handle_security_mode']
        args_dict = {
            'archive_unpack': { 'dir_moves' : dir_moves },
            'handle_security_mode' : { 'files': secure_files }
            }
        dl.do_upgrade_actions(site, site_conf, instance_conf,
                              old_instance_conf, msgs,
                              globals(), actions, args_dict)
	if msgs.get_severity() != msgs.ERROR:
		variables = [('db_login', site_conf['mysql']['dbaseadmin']),
                 		('db_password', dbpassword),
                 		('db_host', dl.get_mysql_host(site)),
                 		('db_database', dbdatabase) ]
                actions = ['edit_conf_file','edit_conf_file1', 'exec_php_script','remove_files','handle_security_mode']
                args_dict = {'edit_conf_file': { 'file':'includes/settings.php','lang':'apache_config',
                               'pat' : pat, 'sub': sub, 'vars': variables },
			'edit_conf_file1': { 'file':'convert_passwords.php','lang':'php',
                               'pat' : pat1, 'sub': sub1, 'vars': variables },
			'exec_php_script' : {'script' : 'convert_passwords.php',
					'args':{},
					'success_re': '.*Finished converting passwords.*'},
			'remove_files': { 'files': ['convert_passwords.php'] },
                        'handle_security_mode': {'files': secure_files }
                        }
                dl.do_upgrade_actions(site, site_conf, instance_conf,
                                        old_instance_conf, msgs,
                                        globals(), actions, args_dict)
        
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Redirecting to ...')
